/*
Jonathan Lamb
jonlambusn@gmail.com
*/

#include "CotsBot.h"

CotsBot::CotsBot(){
	val = BT_FLAG;
	motorSpeed = BOTSPEED;		//change this value in .h to speed up or slow down the movements	
	val = BT_FLAG;				//initialize the serial data to a known value
	bump = false;				//clear bump flag
}

void CotsBot::init(){
	//pinMode(IRsensor,INPUT);	//set pin 2 as input
	pinMode(M1,OUTPUT);			//set pin 4 as output
	pinMode(E1,OUTPUT);			//set pin 5 as output
	pinMode(E2,OUTPUT);			//set pin 6 as output
	pinMode(M2,OUTPUT);			//set pin 7 as output
	pinMode(audioOut,OUTPUT);	//set pin 10 as output
	pinMode(StatusLed,OUTPUT);	//set pin 11 as output
	pinMode(ErrorLed,OUTPUT);	//set pin 12 as output
	//pinMode(switchS7,INPUT);	//set pin 3 as input
	//pinMode(hrdwrLed,OUTPUT);	//set pin 13 as output
	digitalWrite(StatusLed,LOW);//turn of the status led
	digitalWrite(ErrorLed,LOW);	//turn of the status led
	digitalWrite(audioOut,LOW);	//turn the speaker off


	/* Doesn't like the interrupt attachment in here for some reason, not really too big a deal but useful to have an interrupt instead of polling */
	//attachInterrupt(InterruptPin,intrptFnct1,LOW);	//setupt the interrupt on pin 2, when the line goes LOW the IntrptFnct1 will be called

	makeSound();				//audio feedback for initialization, uncomment if you have a speaker connected
	makeSound();	
	Serial.begin(BAUD_RATE);	//setupt the serial connection at the given speed
	delay(500);					//let everything 'settle'
}

char CotsBot::getData(){
	if(Serial.available())		//if the serial data is ready set val
		val = Serial.read();
	else						//if it's not available, set to flag var
		val = BT_FLAG;
	return val;
}

void CotsBot::readSerial(){
	if(Serial.available())		//if the serial data is ready set val
		val = Serial.read();
	else						//if it's not available, set to flag
		val = BT_FLAG;
}

/* use this if you want to send data in char's ( ie. E or R) */
void CotsBot::sendData(char data){
	Serial.print(data);	//will send the BYTE of data of the serial connection
}

/* use this if you want to instead send data in raw form, must know how to handle serial data in terms of size
void CotsBot::sendData(dataType data){
	Serial.write(data,DATATYPE);	//will send the BYTE of data of the serial connection
}
*/

/* set the speed of the bot */
void CotsBot::setSpeed(char mSpeed){
	motorSpeed = mSpeed;
}

char CotsBot::getSpeed(){
	return motorSpeed;
}

/*
	The movement functions use polarity shifting and pulse width modulation to control the direction and speed of the two DC motors
*/
void CotsBot::forward(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,HIGH);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,HIGH);
}

void CotsBot::backward(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,LOW);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,LOW);
}

void CotsBot::left(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,HIGH);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,LOW);
}

void CotsBot::right(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,LOW);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,HIGH);
}

void CotsBot::stop(){
	digitalWrite(E1,LOW);
	digitalWrite(E2,LOW);
}

/* 
---->example for using interrupts, uncomment to use
void CotsBot::intrptFnct1(){
	//NOTE: delay() can't be called inside an interrupt called fx
	//NOTE: should delcare an variables that are modified within an intertupt fx as volatile
	backward();		//move backward until the interrupt is no longer triggered
}
*/

// IR digital sensor on defined PIN, uncomment if this is your setup, in .h too
bool CotsBot::getIR(){
	if(digitalRead(IRsensor)==LOW)
		return true;
	else
		return false;
}

/* This method causes the bot to back up until the sensor no longer detects anything, hardware override to avoid things
void CotsBot::checkIR(){
	while(digitalRead(IRsensor)==LOW){
		bump = true;
		digitalWrite(ErrorLed,HIGH);
		analogWrite(audioOut,audioPWM);
		backward();
		delay(PWM_TIME*3);
	}
	stop();
	digitalWrite(ErrorLed,LOW);
	digitalWrite(audioOut,LOW);
}
*/

void CotsBot::makeSound(){
	analogWrite(audioOut,audioPWM);
	delay(audioLengthMS);
	digitalWrite(audioOut,LOW);
}

/*
This is where all the work is done, Bluetooth mode, add your own fx's here to do other stuff (check sensor's etc)
*/
void CotsBot::recvCmnd(){
	readSerial();				//get the serial BT data
	if(val != BT_FLAG){
		digitalWrite(StatusLed,HIGH);	//turn on the status LED when an action is received
		switch(val){
			case FWRD:	//move forward
				forward();
				break;
			case BKWD:	//move backward
				backward();
				break;
			case LEFT:	//turn left
				left();
				break;
			case RGHT:	//turn right
				right();
				break;
		}
		delay(PWM_TIME);	//defined above
	}
	else{
		stop();
		digitalWrite(ErrorLed,HIGH);	//turn on the error led when no action is received
	}
	digitalWrite(StatusLed,LOW);		//turn off
	digitalWrite(ErrorLed,LOW);			//turn off
	val = BT_FLAG;						//clear the serial data
}


