/*
Jonathan Lamb
jonlambusn@gmail.com
*/

#ifndef CotsBot_h
#define CotsBot_h

#if defined(ARDUINO) && ARDUINO >= 100
  #include "Arduino.h"
#else
  #include "WProgram.h"
#endif
#include <avr/interrupt.h> 

/*#define CotsBot_VERSION 1		software version	*/
const int PWM_TIME = 100;		//a delay in ms, sets the delay for each loop which in turn controls how long each action is active
const int BAUD_RATE = 9600;		//Serial data rate, for standard bluetooth modules its 9600
const int M1 = 4;				//Motor 1 direction contro
const int M2 = 7;				//Motor 2 direction control
const int E1 = 5;				//Motor 1 speed control
const int E2 = 6;				//Motor 2 speed control
const int IRsensor = 9;			//Infrared sensor signal line on pin 2
//const int switchS7 = 3;		//hardware switch 7 (use jumper) on pin 3
const int InterruptPin = 0;		//interrupt 0 is on pin 2, interrupt 1 is on pin 3
const int StatusLed = 11;		//green status LED (or any LED) on pin 11
const int ErrorLed = 12;		//red error status LED on pin 12
const int audioOut = 10;		//speaker/audio output on pin 10
//const int hrdwrLed = 13;		//hardware led on pin 13
const int audioLengthMS = 200;	//length of beep
const int audioPWM = 200;		//PWM value for the speaker
const char FWRD = 0x46;			//forward command, char 'F'
const char BKWD = 0x42;			//backward command, char 'B'
const char LEFT = 0x4C;			//left command, char 'L'
const char RGHT = 0x52;			//right command, char 'R'
const int BT_FLAG = -1;			//the value of a NO command byte
const int BOTSPEED = 0xF1;		//speed of the bot's motors

class CotsBot{
	public:
		CotsBot();								//default constructor
		void init();							//initialize everything, often a lot of this is done is setup()
		char getData();							//if serial data is available then return it, else return -1
		void readSerial();						//read serial data if available, set val value, set to -1 if no data is available
		void sendData(char);					//Send a BYTE of data over the serial connection can be a hex number(0xFF) or a char ('f')
		void setSpeed(char);					//set the speed the motors will use
		char getSpeed();						//get the speed value the motors are using		
		void forward();							//drive the robot forward with a given speed
		void backward();						//drive the robot backward with a given speed
		void left();							//turn the robot left with a given speed
		void right();							//turn the robot right with a given speed
		void stop();							//set both motors to LOW or not moving
		void intrptFnct1();						//function for interrupt on pin 2 (or interrupt0)
		bool getIR();							//check the status of the IR sensor
//		void checkIR();							//if if IR sensor is low call a defined fx/Serial command
		void makeSound();						//create a beep with the predefined length and intensity
		void recvCmnd();						//this function will do all the work, receive a serial command and do the action
//		void checkMode();						//check the status of switch 7 to switch to Bluetooth mode or DEMO/Auto mode
	private:
		char val;								//the BYTE to store the serial data
		char motorSpeed;						//speed for the motors, MIN:0x00 MAX:0xFF
		bool bump;								//flag set when IR sensor is tripped		
};


#endif //CotsBot_h
